﻿// <%STEMCELL:PerDirectoryNoticeText%>
// ALL RIGHTS RESERVED:
// ====================
// The contents of this file, and associated files in this directory, are
// Copyright (C) ThinkGeo,LLC , all rights reserved, 2003-2010.
//
// All software Source Code, Images, Database-Design and code, Graphics Design
// and source files, and related content (collectively referred to as SOURCE)
// are Copyright (c) ThinkGeo,LLC, 2003-2010, All Rights Reserved.
// ThinkGeo,LLC is a USA corporation at 1617 St. Andrews Dr.Suite 201,Lawrence,
// KS 66047.
// http://ThinkGeo.com
//
// MapSuite is a Registered Trademark of ThinkGeo,LLC.
//
//</%STEMCELL:PerDirectoryNoticeText%>

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Globalization;
using System.IO;
using System.Diagnostics;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace CSDesktopSample
{
    public partial class WorldMapKitSample : Form
    {
        private WorldMapKitRenderLayer worldMapKitRenderLayer;
        private bool isMouseDown;

        public WorldMapKitSample()
        {
            InitializeComponent();
        }

        private void FormRenderUSA_Load(object sender, EventArgs e)
        {
            Stopwatch stopwatch = new Stopwatch();
            stopwatch.Start();

            string worldMapKitDataFolder = ConfigurationManager.AppSettings.Get("WorldMapKitDataFolder");
            worldMapKitRenderLayer = new WorldMapKitRenderLayer(worldMapKitDataFolder);

            LayerOverlay worldMapKitRenderLayerOverlay = new LayerOverlay();          
            worldMapKitRenderLayerOverlay.Layers.Add(worldMapKitRenderLayer);
            Map.Overlays.Add(worldMapKitRenderLayerOverlay);

            Map.MapUnit = GeographyUnit.DecimalDegree;
            Map.ExtentOverlay.ZoomPercentage = 50;
            Map.CurrentExtent = WorldMapKitRenderLayer.DefaultExtent;

            // Map.GeoCanvas = new WicBitmapGeoCanvas();
            Map.GeoCanvas = new MyWicCanvas();
            MyWicCanvas.layerCount = worldMapKitRenderLayer.StaticLayers.Count +
                                     worldMapKitRenderLayer.DynamicLayers.Count;

            
            Map.Refresh();

            stopwatch.Stop();
            Debug.WriteLine("GeoCanvas=GDI+ - WorldkMapKit: " + stopwatch.Elapsed.TotalMilliseconds + " ms");
        }

        private void Map_MouseMove(object sender, MouseEventArgs e)
        {
            PointShape WorldPointR = ExtentHelper.ToWorldCoordinate(Map.CurrentExtent, e.X, e.Y, Map.Width, Map.Height);
            if (WorldPointR.X > -180 && WorldPointR.X < 180 && WorldPointR.Y > -90 && WorldPointR.Y < 90)
            {
                PointShape worldPoint = ExtentHelper.ToWorldCoordinate(Map.CurrentExtent, e.X, e.Y, Map.Width, Map.Height);
                StatusBar.Panels[0].Text = DecimalDegreesHelper.GetDegreesMinutesSecondsStringFromDecimalDegreePoint(worldPoint);
                StatusBar.Panels[1].Text = "X: " + String.Format(CultureInfo.InvariantCulture, "{0:#0.0000}", worldPoint.X) + "  Y: " + String.Format(CultureInfo.InvariantCulture, "{0:#0.0000}", worldPoint.Y);
            }
            else
            {
                StatusBar.Panels[0].Text = string.Empty;
                StatusBar.Panels[1].Text = string.Empty;
            }

            StatusBar.Panels[2].Text = "X: " + String.Format(CultureInfo.InvariantCulture, "{0:#0}", e.X) + "  Y: " + String.Format(CultureInfo.InvariantCulture, "{0:#0}", e.Y);
        }

        private void ToolBar_ButtonClick(object sender, ToolBarButtonClickEventArgs e)
        {
            switch (e.Button.Tag.ToString())
            {
                case "Zoom In":
                    Map.ZoomIn(Map.ExtentOverlay.ZoomPercentage);
                    Map.Refresh();
                    break;
                case "Zoom Out":
                    Map.ZoomOut(Map.ExtentOverlay.ZoomPercentage);
                    Map.Refresh();
                    break;
                case "Full Extent":
                    Map.CurrentExtent = WorldMapKitRenderLayer.DefaultExtent;
                    Map.Refresh();
                    break;
                case "Pan Left":
                    Map.Pan(PanDirection.Left, 40); Map.Refresh();
                    break;
                case "Pan Right":
                    Map.Pan(PanDirection.Right, 40);
                    Map.Refresh();
                    break;
                case "Pan Up":
                    Map.Pan(PanDirection.Up, 40);
                    Map.Refresh();
                    break;
                case "Pan Down":
                    Map.Pan(PanDirection.Down, 40);
                    Map.Refresh();
                    break;
                case "Previous Extent":
                    Map.ZoomToPreviousExtent();
                    Map.Refresh();
                    break;
                case "Toggle Extent":
                    Map.ToggleMapExtents();
                    Map.Refresh();
                    break;
                default:
                    break;
            }
        }

        private void chkRenderMap_CheckedChanged(object sender, EventArgs e)
        {
            int count = Map.Overlays.Count;
            for (int i = count -1; i >=0; i--)
            {
                Overlay overlay = Map.Overlays[i];
                try
                {
                    overlay.Lock.EnterWriteLock();

                    if (overlay is LayerOverlay)
                    {
                        ((LayerOverlay)overlay).Layers.Clear();
                    }
                }
                finally
                {
                    overlay.Lock.ExitWriteLock();
                }
            }

            if (chkShowPOI.CheckState == CheckState.Checked)
            {
                worldMapKitRenderLayer.ShowPOI = true;
            }
            else
            {
                worldMapKitRenderLayer.ShowPOI = false;
            }

            LayerOverlay layerOverlay1 = (LayerOverlay)Map.Overlays[0];
            worldMapKitRenderLayer.LoadLayers();
            layerOverlay1.Layers.Add(worldMapKitRenderLayer);
            Map.Refresh();
        }

        private delegate void ChangeCursor();
        private void Map_OverlaysDrawing(object sender, ThinkGeo.MapSuite.DesktopEdition.OverlaysDrawingWinformsMapEventArgs e)
        {
            ChangeCursor changeCursor = new ChangeCursor(ChangeCursorBeforeDrawing);
            this.BeginInvoke(changeCursor);
        }

        private void ChangeCursorBeforeDrawing()
        {
            this.Cursor = Cursors.WaitCursor;
        }

        void ExtentOverlay_MapMouseUp(object sender, MapMouseUpInteractiveOverlayEventArgs e)
        {
            isMouseDown = false;
        }

        void ExtentOverlay_MapMouseDown(object sender, MapMouseDownInteractiveOverlayEventArgs e)
        {
            isMouseDown = true;
        }

        private void Map_OverlaysDrawn(object sender, ThinkGeo.MapSuite.DesktopEdition.OverlaysDrawnWinformsMapEventArgs e)
        {
            ChangeCursor changeCursor = new ChangeCursor(ChangeCursorAfterDrawing);
            this.BeginInvoke(changeCursor);
        }

        private void ChangeCursorAfterDrawing()
        {
            this.Cursor = Cursors.Default;

            double totalRenderTime = GetTotalDrawingTime(Map);
            StatusBar.Panels[3].Text = string.Format(CultureInfo.InvariantCulture, "Total render time: {0:F4}", totalRenderTime);

            if (!isMouseDown)
            {
                UpdateDrawingLayersInfo(worldMapKitRenderLayer, totalRenderTime, listDrawTime);
            }
        }

        private static double GetTotalDrawingTime(WinformsMap winformsMap)
        {
            TimeSpan time = new TimeSpan();
            foreach (Overlay overlay in winformsMap.Overlays)
            {
                if (overlay is LayerOverlay)
                {
                    foreach (Layer layer in ((LayerOverlay)overlay).Layers)
                    {
                        time = time.Add(layer.DrawingTime);
                    }
                }
            }

            return time.TotalSeconds;
        }

        private static void UpdateDrawingLayersInfo(WorldMapKitRenderLayer renderMap, double totalRenderTime, ListView listDrawTime)
        {
            List<LayerRenderTimeInfo> renderLayerLists = new List<LayerRenderTimeInfo>();
            AddRenderLayerToList(renderMap.StaticLayers, renderLayerLists);
            AddRenderLayerToList(renderMap.DynamicLayers, renderLayerLists);
            renderLayerLists.Sort();

            listDrawTime.View = View.Details;
            listDrawTime.Items.Clear();
            double percentage;
            ListViewItem item;
            foreach (LayerRenderTimeInfo rec in renderLayerLists)
            {
                if (rec.dDrawingTime < 0.001)
                {
                    continue;
                }
                percentage = rec.dDrawingTime / totalRenderTime * 100;
                item = listDrawTime.Items.Add(percentage.ToString("N3", CultureInfo.InvariantCulture));
                item.SubItems.Add(rec.dDrawingTime.ToString("N3", CultureInfo.InvariantCulture));
                item.SubItems.Add(rec.strName);
                item.SubItems.Add(rec.shpFileName);
            }
        }

        private static void AddRenderLayerToList(Collection<WorldMapKitShapeFileFeatureLayer> layers, List<LayerRenderTimeInfo> renderLayerLists)
        {
            foreach (Layer layer in layers)
            {
                string shapeFileName = string.Empty;
                ShapeFileFeatureLayer shapeFileFeatureLayer = layer as ShapeFileFeatureLayer;
                if (shapeFileFeatureLayer != null)
                {
                    shapeFileName = shapeFileFeatureLayer.ShapePathFileName;
                    FileInfo fileInfo = new FileInfo(shapeFileName);
                    shapeFileName = fileInfo.Name;
                }

                LayerRenderTimeInfo tmpRenderLayer = new LayerRenderTimeInfo(layer.Name, layer.DrawingTime.TotalSeconds, shapeFileName);
                renderLayerLists.Add(tmpRenderLayer);
            }
        }
    }

    internal struct LayerRenderTimeInfo : IComparable<LayerRenderTimeInfo>
    {
        internal string strName;
        internal double dDrawingTime;
        internal string shpFileName;

        internal LayerRenderTimeInfo(string name, double drawingTime, string shpFileName)
        {
            strName = name;
            dDrawingTime = drawingTime;
            this.shpFileName = shpFileName;
        }

        public int CompareTo(LayerRenderTimeInfo other)
        {
            if (dDrawingTime > other.dDrawingTime)
            {
                return -1;
            }
            if (dDrawingTime < other.dDrawingTime)
            {
                return 1;
            }
            return 0;
        }

        public override string ToString()
        {
            return string.Format(CultureInfo.InvariantCulture, "{0}'s Render Time:{1}", strName, dDrawingTime.ToString(CultureInfo.InvariantCulture));
        }
    }
}
